@@ -4,35 +4,44 @@
 `estimate.multigroup` <- function(x, control=list(),
                                   estimator="gaussian",
                                   weight, weightname,
-                                  weight2, 
+                                  data2, 
                                   cluster=NULL,
                                   silent=lava.options()$silent,
                                   quick=FALSE,
+                                  param,
                                   ...) {
   cl <- match.call()
   optim <- list(
-                iter.max=lava.options()$iter.max,
-                trace=ifelse(lava.options()$debug,3,0),
-                gamma=1,
-                gamma2=1,
-                lambda=0.05,
-                abs.tol=1e-9,
-                epsilon=1e-10,
-                delta=1e-10,
-                S.tol=1e-6,
-                stabil=FALSE,
-                start=NULL,
-                constrain=lava.options()$constrain,
-                method=NULL,
-                starterfun=startvalues,
-                information="E",
-                meanstructure=TRUE,
-                sparse=FALSE,
-                lbound=1e-9,
-                reindex=FALSE,
-                tol=1e-9)
+             iter.max=lava.options()$iter.max,
+             trace=ifelse(lava.options()$debug,3,0),
+             gamma=lava.options()$gamma,
+             ngamma=lava.options()$ngamma,
+             gamma2=1,
+             lambda=0.05,
+             abs.tol=1e-9,
+             epsilon=1e-10,
+             delta=1e-10,
+             S.tol=1e-6,
+             stabil=FALSE,
+             start=NULL,
+             constrain=lava.options()$constrain,
+             method=NULL,
+             starterfun=startvalues,
+             information="E",
+             meanstructure=TRUE,
+             sparse=FALSE,
+             lbound=1e-9,
+             reindex=FALSE,
+             tol=lava.options()$tol)
+
+
+
+  if (!missing(param)) {
+    oldparam <- lava.options()$param
+    lava.options(param=param)
+    on.exit(lava.options(param=oldparam))
+  }
 
-  
   defopt <- lava.options()[]
   defopt <- defopt[intersect(names(defopt),names(optim))]
   optim[names(defopt)] <- defopt
@@ -51,7 +60,7 @@
       pname <- names(optim$start)
       ppos <- parpos.multigroup(x,p=pname,mean=TRUE)
       if (any(!is.na(ppos)))
-        mystart[match(pname,ppos)] <- optim$start[na.omit(match(ppos,pname))]
+        mystart[ppos] <- optim$start[na.omit(match(attributes(ppos)$name,pname))]
     }
     if (!silent) cat("\n")
   }
@@ -90,34 +99,34 @@
   } else {
     weight <- NULL
   }
-  if (!missing(weight2)) {
-    if (is.character(weight2)) {      
-      stweight2 <- weight2
-      weight2 <- list()
+  if (!missing(data2)) {
+    if (is.character(data2)) {      
+      stdata2 <- data2
+      data2 <- list()
       for (i in 1:length(x$data)) {
-        newweight <- as.matrix(x$data[[i]][,stweight2,drop=FALSE])
+        newweight <- as.matrix(x$data[[i]][,stdata2,drop=FALSE])
         dropcol <- apply(newweight,2,function(x) any(is.na(x)))
         newweight <- newweight[,!dropcol,drop=FALSE]
         colnames(newweight) <- index(x$lvm[[i]])$endogenous[seq_len(ncol(newweight))]
-        weight2 <- c(weight2, list(newweight))
+        data2 <- c(data2, list(newweight))
       }
     }
   } else {
-    weight2 <- NULL
+    data2 <- NULL
   }
 
 ### Run hooks (additional lava plugins)
   myhooks <- gethook()
   newweight <- list()
-  newweight2 <- list()
+  newdata2 <- list()
   newoptim <- newestimator <- NULL
   for (f in myhooks) {
     for ( i in seq_len(x$ngroup)) {
-      res <- do.call(f, list(x=x$lvm[[i]],data=x$data[[i]],weight=weight[[i]],weight2=weight2[[i]],estimator=estimator,optim=optim))
+      res <- do.call(f, list(x=x$lvm[[i]],data=x$data[[i]],weight=weight[[i]],data2=data2[[i]],estimator=estimator,optim=optim))
       if (!is.null(res$x)) x$lvm[[i]] <- res$x
       if (!is.null(res$data)) x$data[[i]] <- res$data
       if (!is.null(res$weight)) newweight <- c(newweight,list(res$weight))
-      if (!is.null(res$weight2)) newweight2 <- c(newweight2,list(res$weight2))
+      if (!is.null(res$data2)) newdata2 <- c(newdata2,list(res$data2))
       if (!is.null(res$optim)) newoptim <- res$optim
       if (!is.null(res$estimator)) newestimator <- res$estimator
     }
@@ -127,11 +136,24 @@
       if (!any(unlist(lapply(newweight,is.null)))) {
         weight <- newweight
       }
-    if (!is.null(res$weight2))
-      if (!any(unlist(lapply(newweight2,is.null)))) {
-        weight2 <- newweight2
+    if (!is.null(res$data2))
+      if (!any(unlist(lapply(newdata2,is.null)))) {
+        data2 <- newdata2
       }    
   }
+
+
+  checkestimator <- function(x,...) {
+    ffname <- paste(x,c("_objective","_gradient"),".lvm",sep="")
+    exists(ffname[1])||exists(ffname[2])
+  }
+  if (!checkestimator(estimator)) { ## Try down/up-case version
+    estimator <- tolower(estimator)
+    if (!checkestimator(estimator)) {
+      estimator <- toupper(estimator)
+    }
+  }
+  
   Method <-  paste(estimator, "_method", ".lvm", sep="")
   if (!exists(Method))
     Method <- "nlminb1"
@@ -149,7 +171,6 @@
     data0 <- x$data[[i]]
     xfix0 <- colnames(data0)[(colnames(data0)%in%parlabels(x0,exo=TRUE))]
     xconstrain0 <- intersect(unlist(lapply(constrain(x0),function(z) attributes(z)$args)),manifest(x0))
-    ##    xconstrain <- c(xconstrain,list(xconstrain0))
     xfix <- c(xfix, list(xfix0))
     if (length(xfix0)>0) Xfix<-TRUE ## Yes, random slopes      
     if (length(xconstrain0)>0) Xconstrain <- TRUE ## Yes, nonlinear regression
@@ -192,8 +213,6 @@
 
   InformationFun <- paste(estimator, "_hessian", ".lvm", sep="")
   
-  ##  suppressMessages(browser()) 
-  ##parord <- modelPar(x,1:length(mystart),debug=debug)$p
   parord <- modelPar(x,1:with(x,npar+npar.mean))$p
   mymodel <- x
 
@@ -202,10 +221,8 @@
   myfix <- list()
 
   if (Xfix |  (Xconstrain & XconstrStdOpt | !lava.options()$test)) { ## Model with random slopes:
-################################################################################
-################################################################################
-################################################################################
-
+#############################################################
+    
     if (Xfix) {
       myclass <- c(myclass,"lvmfit.randomslope")
       for (k in 1:x$ngroup) {
@@ -227,7 +244,7 @@
         index(x0) <- reindex(x0,zeroones=TRUE,deriv=TRUE)
         x$lvm[[k]] <- x0
         yvars <- endogenous(x0)
-        parkeep <- c(parkeep, parord[[k]][coef(x1,mean=TRUE)%in%coef(x0,mean=TRUE)])
+        parkeep <- c(parkeep, parord[[k]][coef(x1,mean=TRUE,fix=FALSE)%in%coef(x0,mean=TRUE,fix=FALSE)])
       }
       parkeep <- sort(unique(parkeep))
       ## Alter start-values:
@@ -238,7 +255,6 @@
       x <- multigroup(x$lvm,x$data,fix=FALSE,exo.fix=FALSE) 
     }  
 
-
     parord <- modelPar(x,1:length(mystart))$p    
     mydata <- list()
     for (i in 1:x$ngroup) {      
@@ -251,7 +267,6 @@
       pp <- modelPar(x,theta)$p
       res <- 0
     for (k in 1:x$ngroup) {
-##      for (k in 5:5) {
         x0 <- x$lvm[[k]]
         data0 <- x$data[[k]]
         if (Xfix) {
@@ -266,17 +281,17 @@
               index(x0)$A[cbind(myfix0$row[[i]],myfix0$col[[i]])] <-
                 data0[ii,xfix0[i]]
           }
-          if (is.list(weight2[[k]][ii,])) {
+          if (is.list(data2[[k]][ii,])) {
             res <- do.call(ObjectiveFun, list(x=x0, p=p0,
                                               data=data0[ii,manifest(x0),drop=FALSE],
                                               n=1, S=NULL, weight=weight[[k]][ii,],
-                                              weight2=weight2[[k]]))
+                                              data2=data2[[k]]))
             
           } else {
             res <- do.call(ObjectiveFun, list(x=x0, p=p0,
                                               data=data0[ii,manifest(x0),drop=FALSE],
                                               n=1, S=NULL, weight=weight[[k]][ii,],
-                                              weight2=weight2[[k]][ii,]))
+                                              data2=data2[[k]][ii,]))
           }
           return(res)
         }
@@ -303,14 +318,14 @@
               index(x0)$A[cbind(myfix0$row[[i]],myfix0$col[[i]])] <-
                 x$data[[k]][ii,xfix[[k]][i]]
           }
-          if (is.list(weight2[[k]][ii,])) {
+          if (is.list(data2[[k]][ii,])) {
             
           } else {
             val <- do.call(GradFun, list(x=x0, p=pp[[k]],
                                          data=mydata[[k]][ii,,drop=FALSE], n=1,
                                          S=NULL,
                                          weight=weight[[k]][ii,],
-                                         weight2=weight2[[k]][ii,]))
+                                         data2=data2[[k]][ii,]))
           }
           return(val)
         }
@@ -348,7 +363,7 @@
                             data=mydata[[k]][ii,], n=1,
                             S=NULL,
                             weight=weight[[k]][ii,],
-                            weight2=weight2[[k]][ii,],
+                            data2=data2[[k]][ii,],
                             type=optim$information
                             )
                        )
@@ -358,7 +373,6 @@
             attributes(I)$grad <- D
           }
           I[ parord[[k]], parord[[k]] ] <- J
-##                                                   list(p=pp[[k]], model=x0, data=x$data[[k]], n=1, weight=weight[[k]][ii,]))
           return(I)
         }      
         L <- lapply(1:nrow(x$data[[k]]),function(x) myfun(x))
@@ -371,9 +385,7 @@
       return(res)   
     }
   } else { ## Model without random slopes:    
-################################################################################
-#########################################2#######################################
-################################################################################
+###########################################################
 
 
     ## Non-linear parameter constraints involving observed variables? (e.g. nonlinear regression)
@@ -425,8 +437,7 @@
       return(NULL)
     }
 
-
-    
+   
     myObj <- function(theta) {     
       theta0 <- theta
       if (optim$constrain) {
@@ -450,7 +461,7 @@
           x0$mean[yconstrain[[i]]] <- 0 
         }
         res <- c(res,
-                 do.call(ObjectiveFun, list(x=x0, p=pp[[i]], data=data0, S=S, mu=mu, n=n, weight=weight[[i]], weight2=weight2[[i]], offset=offset)))
+                 do.call(ObjectiveFun, list(x=x0, p=pp[[i]], data=data0, S=S, mu=mu, n=n, weight=weight[[i]], data2=data2[[i]], offset=offset)))
                  
       }
         sum(res)
@@ -471,7 +482,7 @@
                          do.call(GradFun, list(x=x$lvm[[i]],p=pp[[i]],
                                                data=x$data[[i]][,index(x$lvm[[i]])$manifest,drop=FALSE],
                                                S=S,mu=mu,n=n,
-                                               weight=weight[[i]], weight2=weight2[[i]])))
+                                               weight=weight[[i]], data2=data2[[i]])))
           D <- D0; D[ parord[[i]] ] <- repval
         res <- res + D
         }
@@ -487,16 +498,14 @@
       if (optim$constrain) {
         theta[constrained] <- exp(theta[constrained])
       }
-      ##      parord <- modelPar(x,1:length(theta),debug=debug)$p
       pp <- modelPar(x,theta)$p
       I0 <- res <- matrix(0,length(theta),length(theta))
       for (i in 1:x$ngroup) {
         I <- I0;
         I[ parord[[i]], parord[[i]] ] <- with(x$samplestat[[i]], do.call(InformationFun, list(p=pp[[i]], x=x$lvm[[i]], data=x$data[[i]],
                                                                                               S=S, mu=mu, n=n, weight=weight[[i]],
-                                                                                              weight2=weight2[[i]],
+                                                                                              data2=data2[[i]],
                                                                                               type=optim$information)))
-        ##with(x$samplestat[[i]], information(x$lvm[[i]],p=pp[[i]],n=n))
         res <- res + I
       }
       D <- myGrad(theta0)
@@ -514,10 +523,9 @@
     }    
   }
   
-################################################################################
-################################################################################
-################################################################################
+##############################################################
 
+  
   if (!exists(InformationFun)) myInformation <- NULL
   else if (is.null(get(InformationFun))) myInformation <- NULL
   if (is.null(get(GradFun))) myGrad <- NULL
@@ -554,22 +562,18 @@
   if (is.null(myInformation)) {
     if (!require("numDeriv")) stop("I do not know how to calculate the asymptotic variance of this estimator.
 For numerical approximation please install the library 'numDeriv'.")
-    ##    cat("Using a numerical approximation of hessian...\n");
     if (!is.null(myGrad) & XconstrStdOpt)
       myInformation <- function(theta) jacobian(myGrad, theta, method=lava.options()$Dmethod)
     else {
-      myInformation <- function(theta) -hessian(myObj, theta)
+      myInformation <- function(theta) hessian(myObj, theta)
     }
   }
 }
   I <- myInformation(opt$estimate)
   asVar <- tryCatch(Inverse(I),
                     error=function(e) matrix(NA, length(mystart), length(mystart)))
-
-##  if (!silent) cat("\n")
-
     
-  res <- list(model=x, model0=mymodel, call=cl, opt=opt, meanstructure=optim$meanstructure, vcov=asVar, estimator=estimator, weight=weight, weight2=weight2, cluster=cluster)
+  res <- list(model=x, model0=mymodel, call=cl, opt=opt, meanstructure=optim$meanstructure, vcov=asVar, estimator=estimator, weight=weight, data2=data2, cluster=cluster)
   class(res) <- myclass
 
   myhooks <- gethook("post.hooks")
