@@ -1,3 +1,5 @@
+rsq <- function(x) 1-coef(x,9,std="yx")[paste(manifest(x),"<->",manifest(x),sep=""),5] 
+
 satmodel <- function(object,logLik=TRUE,data=model.frame(object),
 ##                     control=list(start=coef(object),trace=1),
                      control=list(trace=1),
@@ -70,6 +72,7 @@ condition <- function(A) {
 ##' @param x Model object
 ##' @param p Parameter vector used to calculate statistics
 ##' @param data Data.frame to use
+##' @param data2 Optional second data.frame (only for censored observations)
 ##' @param weight Optional weight matrix
 ##' @param n Number of observations
 ##' @param conditional If TRUE the conditional moments given the covariates are
@@ -97,11 +100,12 @@ condition <- function(A) {
 ##'                       data=model.frame(object),
 ##'                       model=object$estimator,
 ##'                       weight=Weight(object),
+##'                       data2=object$data$data2,
 ##'                           ...)
 ##' 
-##' \method{score}{lvmfit}(x, data=model.frame(x), p=pars(x), model=x$estimator, weight=Weight(x), ...)
+##' \method{score}{lvmfit}(x, data=model.frame(x), p=pars(x), model=x$estimator, weight=Weight(x), data2=x$data$data2, ...)
 ##' 
-##' \method{information}{lvmfit}(x,p=pars(x),n=x$data$n,data=model.frame(x),model=x$estimator,weight=Weight(x),...)
+##' \method{information}{lvmfit}(x,p=pars(x),n=x$data$n,data=model.frame(x),model=x$estimator,weight=Weight(x), data2=x$data$data2, ...)
 ##' 
 ##' @return A \code{htest}-object.
 ##' @author Klaus K. Holst
@@ -132,7 +136,6 @@ gof.lvmfit <- function(object,chisq=FALSE,level=0.90,rmsea.threshold=0.05,all=FA
   l2D <- sum(object$opt$grad^2)
   rnkV <- tryCatch(qr(vcov(object))$rank,error=function(...) 0)
   condnum <- tryCatch(condition(vcov(object)),error=function(...) NULL)
-  
 
 ##  if (class(object)[1]=="lvmfit" & (object$estimator=="gaussian" | chisq) & length(xconstrain)==0 ) {
   if (((object$estimator=="gaussian" & class(object)[1]!="lvm.missing") | chisq) & length(xconstrain)==0 ) {
@@ -161,8 +164,11 @@ gof.lvmfit <- function(object,chisq=FALSE,level=0.90,rmsea.threshold=0.05,all=FA
       R <- (diag(1/L))%*%(S-C)%*%(diag(1/L))
       R2 <- (mu-xi)/L
       SRMR <- mean(c(R[upper.tri(R,diag=TRUE)],R2)^2)^0.5
-      SRMR.endo <- mean(c(R[idx,idx][upper.tri(R[idx,idx],diag=TRUE)],R2[idx])^2)^0.5      
-      res <- c(res,list(CFI=CFI,NFI=NFI,TLI=TLI,C=C,S=S,SRMR=SRMR,"SRMR(endogenous)"=SRMR.endo))
+      res <- c(res,list(CFI=CFI,NFI=NFI,TLI=TLI,C=C,S=S,SRMR=SRMR))
+      ## if (length(latent(object))>0) {
+      ##   SRMR.endo <- mean(c(R[idx,idx][upper.tri(R[idx,idx],diag=TRUE)],R2[idx])^2)^0.5      
+      ##   res <- c(res,list("SRMR(endogenous)"=SRMR.endo))
+      ## }
     }    
     ##    if (class(object)[1]=="lvmfit")
     if (rnkV==ncol(vcov(object))) {
